#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

/* this provides compatibility with the mace file definitions for knots */
#ifndef DUAL
#  define f(_X_,_Y_) f[_X_][_Y_]
#  define g(_X_,_Y_) g[_X_][_Y_]
#else /* color by dual quandle (f and g switched) */
#  define g(_X_,_Y_) f[_X_][_Y_]
#  define f(_X_,_Y_) g[_X_][_Y_]
#endif


/* to choose a quandle 3, compile the program with -D Q3 or use #define Q3 */
//#define Q13
#include "quandle_list.c"
#define NTHREADS n
int colorvector[n]; /* globally shared array of colorings */

/* NAME: knot()
 * DESCRIPTION: This is the function that is multithreaded.
 *              It performs the actual knot coloring.
 */
void *knot(void *arg)
{
  long int color_count=0;
  int taskid=*(int *)arg;
  /* fix the first strand to 0 since other colorings are isomorphic */
#define a1 0
#define a2 taskid
  int a3,a4,a5,a6,a7; /* strands */

  /* the desired knot is conditionally included by specifying     *
   * -DK<some_number> at compile time. To specify the 200th knot, *
   * at compile time, we pass -DK200 to gcc.                      *
   * Alternatively, one can just insert #define K200 into this    *
   * file before compiling.                                       */
#include "knot_list_individuals_knotid_braid_index.c"


#if defined DEBUG
  printf("thread %04d colorings = %9d\n",taskid,color_count);
#endif

  /* store this thread's result in a globally shared array */
  colorvector[taskid] = color_count;
  return NULL;
}

/******** this is the parent thread's code ********/
int main()
{
  int i,j,k;
  int taskid;
  pthread_t threads[NTHREADS];                /* holds thread info */
  int ids[NTHREADS];
  int errorcode;                              /* pthread error code */
  int *status;                                /* thread return code */
  long int total_colorings;


  /* initialize dual quandle g[n][n] */
  for(i=0;i<n;i++) {
    for(j=0;j<n;j++) {
      g[f[i][j]][j] = i;
    }
  }


  /* create threads */
  for (errorcode=1,taskid=0; taskid<NTHREADS; taskid++) {
    ids[taskid]=taskid;
    errorcode = pthread_create(&threads[taskid], /* thread structure          */
                               NULL,             /* default thread attributes */
                               knot,             /* thread function           */
                               &ids[taskid]);    /* thread argument           */
    if (errorcode) {
      printf("ERROR :: pthread_create() :: code=%d\n",errorcode);
      exit(1);
    }
  }
  /* join threads */
  for (total_colorings=0,errorcode=1,taskid=0; taskid<NTHREADS; taskid++) {
    errorcode = pthread_join(threads[taskid],(void *)&status);
    total_colorings += colorvector[taskid];
    if (errorcode) {
      printf("ERROR :: pthread_join() :: code=%d\n",errorcode);
      exit(1);
    }
  }
  /* Convert the "fixed colorings result" to the real    *
   * number of colorings using the following definition. *
   * real_number_of_colorings = |Q|(total_colorings - 1) */
  total_colorings = n*(total_colorings - 1);
  printf("colorings( q%04d , k%04d ): %16d\n",QUANDLE,knotid,total_colorings);

  return(0);
}
/* EOF */
